/** 
 * @namespace 
 * @name fbauth 
*/

/** @lends fbauth */

var couchlib = require(__dirname + '/couchlib.js');
var lib = require(__dirname + '/lib.js');

/** Check to see if a user already exists. */
var check_for_user = function (couchdb_opts, name, cb) {
  var opts = lib.clone(couchdb_opts);
  opts.views = exports.couchdb_views;
  couchlib.check_object_exists(opts, 'users', name, cb);
};
exports.check_for_user = check_for_user;

exports.couchdb_views = {
  users: [
    {
      uri: '_design/all_users',
      design: {'all': {'map': "function(doc) { emit(doc._id, doc); }"}}
    }
  ]
};

/** Add a user. */
var add_user = function (couchdb_opts, username, displayname, email, password, cb) {
  var opts = lib.clone(couchdb_opts);
  opts.views = exports.couchdb_views;
  var dbu = couchlib.get_couchdb_schema(opts, 'users');
  dbu.save(username, {displayname: displayname, email: email, password: lib.sha1(password)}, function (err, doc) {
    if (err) {
      cb(err, false);
    } else {
      var dbd = couchlib.get_couchdb_schema(opts, 'displaynames');
      dbd.save(displayname, {username: username}, function (err, doc) {
        if (err) {
          var util = require('util');
          console.log('dbu ' + util.inspect(dbu));
          dbu.delete(username, function (errdel) {
            cb(err, false);
          });
        } else {
          cb(undefined, true);
        }
      });
    }
  });
};
exports.add_user = add_user;

/** Load a users document. */
var load_user = function (couchdb_opts, username, cb) {
  var opts = lib.clone(couchdb_opts);
  opts.views = exports.couchdb_views;
  var db = couchlib.get_couchdb_schema(opts, 'users');
  db.get(username, function (err, doc) {
    if (err) {
      cb(err, undefined);
    } else {
      cb(undefined, doc);
    }
  });
};
exports.load_user = load_user;

/** Login a user, returning the document. */
var login_user = function (couchdb_opts, username, password, cb) {
  load_user(couchdb_opts, username, function (err, doc) {
    if (err) {
      cb(err, undefined);
    } else {
      if (lib.sha1(password) == doc.password) {
        cb(undefined, doc);
      } else {
        cb("invalid_username_password", undefined);
      }
    }
  });
};
exports.login_user = login_user;

/** Check if the displayname already exists. */
var check_for_displayname = function (couchdb_opts, name, cb) {
  couchlib.check_object_exists(couchdb_opts, 'displaynames', name, cb);
};
exports.check_for_displayname = check_for_displayname;

/** Load a user based upon displayname. */
var load_user_displayname = function (couchdb_opts, displayname, cb) {
  var opts = lib.clone(couchdb_opts);
  opts.views = exports.couchdb_views;
  var db = couchlib.get_couchdb_schema(opts, 'displaynames');
  db.get(displayname, function (err, doc) {
    if (err) {
      cb(err, undefined);
    } else {
      load_user(doc.username, function (err, doc) {
        if (err) {
          cb(err, undefined);
        } else {
          cb(undefined, doc);
        }
      });
    }
  });
};
exports.load_user_displayname = load_user_displayname;
