(function () {
  var express = require('express');
  var jade = require('jade');
  var couchlib = require(__dirname + '/lib/couchlib.js');
  var filterlib = require(__dirname + '/lib/filterlib.js');
  var fbauth = require(__dirname + '/lib/fbauth.js');
  var lib = require(__dirname + '/lib/lib.js');

  exports.fbauth = fbauth;
  exports.couchlib = couchlib;
  exports.filterlib = filterlib;
  exports.lib = lib;

  var create = function (static_dir, session_key, register_code, base_navs, unauth_navs, authed_navs, couchdb_opts, ssl_opts) {
    if (typeof ssl_opts != 'undefined') {
      var app = express.createServer(ssl_opts);
    } else {
      var app = express.createServer();
    }
    app.news = {};
    
    var loadUser = function (req, res, next) {
      if (typeof req.session.username != 'undefined' && req.session.username.length > 0) {
        var cdb = couchlib.get_couchdb_schema(couchdb_opts, 'users');
        cdb.get(req.session.username, function (err, doc) {
          if (err) {
            req.user = false;
          } else {
            req.user = doc;
          }
          next();
        });
      } else {
        req.user = false;
        next();
      }
    };
    app.loadUser = loadUser;
  
    var getNavs = function (req, res, next) {
      req.navs = lib.clone(base_navs);
      if (req.user == false) {
        for(var i in unauth_navs) {
          if(unauth_navs.hasOwnProperty(i)) {
            req.navs[i] = unauth_navs[i];
          };
        };
      } else {
        for(var i in authed_navs) {
          if(authed_navs.hasOwnProperty(i)) {
            req.navs[i] = authed_navs[i];
          };
        };
      }
      next();
    };
    app.getNavs = getNavs;

    app.configure(function() {
      app.set('view engine', 'jade');
      app.use(express.cookieParser());
      app.use(express.bodyParser());
      app.use(express.static(static_dir));
      app.use(express.responseTime());
      app.use(express.session({secret: session_key}));
    });

    app.get('/', loadUser, getNavs, function (req, res, next) {
      res.render('index.jade', { locals: { navs: req.navs, news: app.news } });
    });

    app.get('/about', loadUser, getNavs, function (req, res, next) {
      res.render('about.jade', { locals: { navs: req.navs } });
    });

    var register_vars = function (navs, message, code, username, displayname, email) {
      return {navs: navs, message: message, code: code, username: username, displayname: displayname, email: email};
    };

    app.get('/register', loadUser, getNavs, function (req, res, next) {
      res.render('register.jade', { locals: register_vars(req.navs, '', '', '', '', '') });
    });

    app.post('/register', loadUser, getNavs, function (req, res, next) {
      var code = filterlib.alphanumeric(req.body.code);
      var username = filterlib.alphanumeric_lower(req.body.username);
      var displayname = filterlib.alphanumeric(req.body.displayname);
      var email = req.body.email;
      var password = req.body.password;
      var password2 = req.body.password2;
      if (register_code.length > 0 && register_code != code) {
        res.render('register.jade', { locals: register_vars(req.navs, 'Invalid code', '', username, displayname, email) });
      } else {
        if (password != password2) {
          res.render('register.jade', { locals: register_vars(req.navs, 'Both password fields should be the same.', code, username, displayname, email) });
        } else {
          fbauth.check_for_user(couchdb_opts, username, function (err, exists) {
            if (err) {
              res.render('register.jade', { locals: register_vars(req.navs, 'Some error occured!', code, username, displayname, email) });
            } else if (exists == true) {
              res.render('register.jade', { locals: register_vars(req.navs, 'User already exists, please choose a different username.', code, username, displayname, email) });
            } else {
              fbauth.check_for_displayname(couchdb_opts, displayname, function (err, exists) {
                if (err) {
                  res.render('register.jade', { locals: register_vars(req.navs, 'Some error occured!', code, username, displayname, email) });
                } else if (exists == true) {
                  res.render('register.jade', { locals: register_vars(req.navs, 'Display name already exists, please choose a different display name.', code, username, displayname, email) });
                } else {
                  fbauth.add_user(couchdb_opts, username, displayname, email, password, function (err, success) {
                    if (err) {
                      res.render('register.jade', { locals: register_vars(req.navs, 'Some error occured!', code, username, displayname, email) });
                    } else {
                      if (success == true) {
                        req.session.username = username;
                        res.redirect('/');
                      } else {
                        res.render('register.jade', { locals: register_vars(req.navs, 'Some error occured!', code, username, displayname, email) });
                      }
                    }
                  });
                }
              });
            }
          });
        }
      }
    });

    app.get('/login', loadUser, getNavs, function (req, res, next) {
      res.render('login.jade', { locals: { navs: req.navs, message: '' } });
    });

    app.post('/login', loadUser, getNavs, function (req, res, next) {
      var username = filterlib.alphanumeric_lower(req.body.username);
      var password = req.body.password;
      fbauth.login_user(couchdb_opts, username, password, function (err, user) {
        if (err) {
          if (err.error == 'not_found') {
            res.render('login.jade', { locals: { navs: req.navs, message: 'Invalid username or password.' } });
          } else {
            if (err == 'invalid_username_password') {
              res.render('login.jade', { locals: { navs: req.navs, message: 'Invalid username or password.' } });
            } else {
              res.render('login.jade', { locals: { navs: req.navs, message: 'Some odd error occured, please tell someone about it.' } });
            }
          }
        } else {
          req.session.username = username;
          res.redirect('/');
        }
      });
    });

    app.get('/logout', function (req, res, next) {
      req.session.destroy();
      res.redirect('/');
    });

    return app;
  };
  exports.create = create;
})();
